/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.log.ui.logdialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

import com.fluxicon.slickerbox.components.DistributionUI;
import com.fluxicon.slickerbox.components.RoundedPanel;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class LogDashboardUI extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6586390798776862988L;
	protected Color colorEnclosureBg = new Color(250, 250, 250, 105);
	protected Color colorTitleFg = new Color(20, 20, 20, 230);
	protected Color colorInfoBg = new Color(60, 60, 60, 160);
	protected Color colorInfoBgMouseOver = new Color(60, 60, 60, 240);
	protected Color colorInfoLabel = new Color(210, 210, 210);
	protected Color colorInfoValue = new Color(255, 255, 255);

	protected SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	protected XLogInfo summary;

	protected int numberOfProcesses = 0;
	protected int numberOfCases = 0;
	protected int numberOfEvents = 0;
	protected int minEventsPerCase = 0;
	protected int maxEventsPerCase = 0;
	protected int meanEventsPerCase = 0;
	protected int[] eventsPerCases = {};
	protected int minEventClassesPerCase = 0;
	protected int maxEventClassesPerCase = 0;
	protected int meanEventClassesPerCase = 0;
	protected int[] eventClassesPerCases = {};
	protected int numberOfEventTypes = 0;
	protected int numberOfEventClasses = 0;
	protected int numberOfOriginators = 0;

	public LogDashboardUI(XLog log, XLogInfo summary) {
		this.summary = summary;
		numberOfProcesses = 1;
		numberOfCases = log.size();
		numberOfEvents = this.summary.getNumberOfEvents();

		numberOfEventTypes = this.summary.getTransitionClasses().size();
		numberOfEventClasses = this.summary.getEventClasses().size();
		numberOfOriginators = this.summary.getResourceClasses().size();
		meanEventsPerCase = 0;
		meanEventClassesPerCase = 0;
		eventClassesPerCases = new int[log.size()];
		eventsPerCases = new int[log.size()];

		for (int i = 0; i < log.size(); i++) {
			int events = log.get(i).size();
			eventsPerCases[i] = events;
			//this.numberOfEvents += events;
			meanEventsPerCase += events;

			int eventClasses = getClassesFor(log.get(i), XLogInfoImpl.STANDARD_CLASSIFIER).size();
			eventClassesPerCases[i] = eventClasses;
			meanEventClassesPerCase += eventClasses;
		}
		if (numberOfEvents == 0) {
			return;
		}
		Arrays.sort(eventsPerCases);
		minEventsPerCase = eventsPerCases[0];
		maxEventsPerCase = eventsPerCases[eventsPerCases.length - 1];
		meanEventsPerCase /= log.size();
		Arrays.sort(eventClassesPerCases);
		minEventClassesPerCase = eventClassesPerCases[0];
		maxEventClassesPerCase = eventClassesPerCases[eventClassesPerCases.length - 1];
		meanEventClassesPerCase /= log.size();

		setupGui();
	}

	private Collection<XEventClass> getClassesFor(XTrace trace, XEventClassifier classifier) {
		XEventClasses classes = new XEventClasses(classifier);
		classes.register(trace);
		return classes.getClasses();
	}

	protected void setupGui() {
		setOpaque(false);
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		setBorder(BorderFactory.createEmptyBorder(1, 2, 4, 4));
		// create key data panel
		RoundedPanel keyPanel = new RoundedPanel(15, 0, 3);
		keyPanel.setMinimumSize(new Dimension(180, 100));
		keyPanel.setMaximumSize(new Dimension(300, 1000));
		keyPanel.setPreferredSize(new Dimension(200, 500));
		keyPanel.setBackground(colorEnclosureBg);
		keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.Y_AXIS));
		keyPanel.add(getLeftAlignedHeader("Key data"));
		keyPanel.add(Box.createVerticalStrut(15));
		keyPanel.add(packInfo("Processes", Integer.toString(numberOfProcesses)));
		keyPanel.add(Box.createVerticalStrut(3));
		keyPanel.add(packInfo("Cases", Integer.toString(numberOfCases)));
		keyPanel.add(Box.createVerticalStrut(3));
		keyPanel.add(packInfo("Events", Integer.toString(numberOfEvents)));
		keyPanel.add(Box.createVerticalStrut(15));
		keyPanel.add(packInfo("Event classes", Integer.toString(numberOfEventClasses)));
		keyPanel.add(Box.createVerticalStrut(3));
		keyPanel.add(packInfo("Event types", Integer.toString(numberOfEventTypes)));
		keyPanel.add(Box.createVerticalStrut(15));
		keyPanel.add(packInfo("Originators", Integer.toString(numberOfOriginators)));
		keyPanel.add(Box.createVerticalGlue());
		// create info panel
		RoundedPanel infoPanel = new RoundedPanel(15, 0, 5);
		infoPanel.setMinimumSize(new Dimension(170, 100));
		infoPanel.setMaximumSize(new Dimension(300, 1000));
		infoPanel.setPreferredSize(new Dimension(190, 500));
		infoPanel.setBackground(colorEnclosureBg);
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.add(getLeftAlignedHeader("Log info"));
		infoPanel.add(Box.createVerticalStrut(12));

		Date time = summary.getLogTimeBoundaries().getStartDate();
		infoPanel.add(getInfoPanel("Start date", (time == null ? "" : time.toString())));
		infoPanel.add(Box.createVerticalStrut(6));
		time = summary.getLogTimeBoundaries().getEndDate();
		infoPanel.add(getInfoPanel("End date", (time == null ? "" : time.toString())));
		// TODO:
//		String description = "Still to make a new XExtension for this";
//		if ((description != null) && (description.length() > 0)) {
//			infoPanel.add(Box.createVerticalStrut(10));
//			infoPanel.add(getInfoPanel("Description", description));
//		}
		infoPanel.add(Box.createVerticalGlue());
		// create action panel
		RoundedPanel actionPanel = new RoundedPanel(15, 0, 0);
		actionPanel.setBackground(new Color(250, 250, 250, 80));
		actionPanel.setLayout(new BoxLayout(actionPanel, BoxLayout.X_AXIS));

		actionPanel.add(Box.createHorizontalGlue());
		JPanel rightPanel = new JPanel();
		rightPanel.setBorder(BorderFactory.createEmptyBorder());
		rightPanel.setMinimumSize(new Dimension(180, 100));
		rightPanel.setMaximumSize(new Dimension(300, 1000));
		rightPanel.setPreferredSize(new Dimension(200, 500));
		rightPanel.setOpaque(false);
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
		rightPanel.add(infoPanel);
		rightPanel.add(Box.createVerticalStrut(10));
		rightPanel.add(actionPanel);
		// right side
		JPanel distributionPanel = new JPanel();
		distributionPanel.setOpaque(false);
		distributionPanel.setBorder(BorderFactory.createEmptyBorder());
		distributionPanel.setLayout(new BoxLayout(distributionPanel, BoxLayout.Y_AXIS));
		distributionPanel.add(getDistributionPanel("Events per case", eventsPerCases, meanEventsPerCase));
		distributionPanel.add(Box.createVerticalStrut(10));
		distributionPanel.add(getDistributionPanel("Event classes per case", eventClassesPerCases,
				meanEventClassesPerCase));
		// assemble
		this.add(keyPanel);
		this.add(Box.createHorizontalStrut(10));
		this.add(distributionPanel);
		this.add(Box.createHorizontalStrut(10));
		this.add(rightPanel);
		revalidate();
	}

	protected RoundedPanel packInfo(String name, String value) {
		RoundedPanel packed = new RoundedPanel(10, 0, 0);
		packed.setBackground(colorInfoBg);
		final RoundedPanel target = packed;
		packed.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) { /* ignore */
			}

			public void mouseEntered(MouseEvent arg0) {
				target.setBackground(colorInfoBgMouseOver);
				target.repaint();
			}

			public void mouseExited(MouseEvent arg0) {
				target.setBackground(colorInfoBg);
				target.repaint();
			}

			public void mousePressed(MouseEvent arg0) { /* ignore */
			}

			public void mouseReleased(MouseEvent arg0) { /* ignore */
			}
		});
		packed.setLayout(new BoxLayout(packed, BoxLayout.X_AXIS));
		JLabel nameLabel = new JLabel(name);
		nameLabel.setOpaque(false);
		nameLabel.setForeground(colorInfoLabel);
		nameLabel.setFont(nameLabel.getFont().deriveFont(12f));
		JLabel valueLabel = new JLabel(value);
		valueLabel.setOpaque(false);
		valueLabel.setForeground(colorInfoValue);
		valueLabel.setFont(valueLabel.getFont().deriveFont(14f));
		packed.add(Box.createHorizontalStrut(5));
		packed.add(nameLabel);
		packed.add(Box.createHorizontalGlue());
		packed.add(valueLabel);
		packed.add(Box.createHorizontalStrut(5));
		packed.revalidate();
		return packed;
	}

	protected RoundedPanel getDistributionPanel(String title, int[] values, int meanValue) {
		// create distribution panel
		RoundedPanel instancePanel = new RoundedPanel(15, 0, 0);
		instancePanel.setBackground(colorEnclosureBg);
		instancePanel.setLayout(new BoxLayout(instancePanel, BoxLayout.Y_AXIS));
		instancePanel.add(getLeftAlignedHeader(title));
		instancePanel.add(Box.createVerticalStrut(6));
		if (values.length == 0) {
			return instancePanel;
		}
		RoundedPanel instanceDistPanel = new RoundedPanel(10, 0, 0);
		instanceDistPanel.setBackground(new Color(20, 20, 20));
		instanceDistPanel.setLayout(new BorderLayout());
		DistributionUI instanceDistUI = new DistributionUI(values);
		instanceDistPanel.add(instanceDistUI, BorderLayout.CENTER);
		JPanel keyPanel = new JPanel();
		keyPanel.setOpaque(false);
		keyPanel.setBorder(BorderFactory.createEmptyBorder());
		keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.X_AXIS));
		keyPanel.add(packInfo("Min", Integer.toString(values[0])));
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(packInfo("Mean", Integer.toString(meanValue)));
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(packInfo("Max", Integer.toString(values[values.length - 1])));
		instancePanel.add(instanceDistPanel);
		instancePanel.add(Box.createVerticalStrut(4));
		instancePanel.add(keyPanel);
		return instancePanel;
	}

	protected JPanel getLeftAlignedHeader(String title) {
		JLabel hLabel = new JLabel(title);
		hLabel.setOpaque(false);
		hLabel.setForeground(colorTitleFg);
		hLabel.setFont(hLabel.getFont().deriveFont(15f));
		return alignLeft(hLabel);
	}

	protected JPanel alignLeft(JComponent component) {
		JPanel hPanel = new JPanel();
		hPanel.setBorder(BorderFactory.createEmptyBorder());
		hPanel.setOpaque(false);
		hPanel.setLayout(new BoxLayout(hPanel, BoxLayout.X_AXIS));
		hPanel.add(component);
		hPanel.add(Box.createHorizontalGlue());
		return hPanel;
	}

	protected JPanel getInfoPanel(String name, String value) {
		JPanel infoPanel = new JPanel();
		infoPanel.setOpaque(false);
		infoPanel.setBorder(BorderFactory.createEmptyBorder());
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		JLabel nameLabel = new JLabel(name);
		nameLabel.setOpaque(false);
		nameLabel.setFont(nameLabel.getFont().deriveFont(10f));
		nameLabel.setForeground(new Color(60, 60, 60));
		JLabel valueLabel = new JLabel("<html>" + value + "</html>");
		valueLabel.setOpaque(false);
		valueLabel.setFont(nameLabel.getFont().deriveFont(12f));
		valueLabel.setForeground(new Color(10, 10, 10));
		infoPanel.add(alignLeft(nameLabel));
		infoPanel.add(Box.createVerticalStrut(2));
		infoPanel.add(alignLeft(valueLabel));
		return infoPanel;
	}

}
